/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.apilib.common.callcontext;

import static de.ozgcloud.common.grpc.GrpcUtil.*;

import java.util.Optional;
import java.util.UUID;

import org.apache.commons.lang3.StringUtils;

import de.ozgcloud.apilib.user.OzgCloudUserId;
import io.grpc.CallOptions;
import io.grpc.Channel;
import io.grpc.ClientCall;
import io.grpc.ClientInterceptor;
import io.grpc.ForwardingClientCall.SimpleForwardingClientCall;
import io.grpc.Metadata;
import io.grpc.MethodDescriptor;
import lombok.RequiredArgsConstructor;

@RequiredArgsConstructor
public class OzgCloudCallContextAttachingInterceptor implements ClientInterceptor {

	static final String KEY_USER_ID = "USER_ID-bin";
	static final String KEY_CLIENT_NAME = "CLIENT_NAME-bin";
	static final String KEY_REQUEST_ID = "REQUEST_ID-bin";
	static final String KEY_ACCESS_LIMITED_ORGAID = "ACCESS_LIMITED_TO_ORGANISATORISCHEEINHEITENID-bin";
	static final String KEY_ACCESS_LIMITED = "ACCESS_LIMITED-bin";

	private final OzgCloudCallContextProvider callContextProvider;

	@Override
	public <A, B> ClientCall<A, B> interceptCall(MethodDescriptor<A, B> method, CallOptions callOptions, Channel next) {
		return new CallContextAttachingClientCall<>(next.newCall(method, callOptions));
	}

	final class CallContextAttachingClientCall<A, B> extends SimpleForwardingClientCall<A, B> {

		protected CallContextAttachingClientCall(ClientCall<A, B> delegate) {
			super(delegate);
		}

		@Override
		public void start(Listener<B> responseListener, Metadata headers) {
			headers.merge(buildCallContextMetadata());
			super.start(responseListener, headers);
		}

		private Metadata buildCallContextMetadata() {
			var ctxt = callContextProvider.provideContext();
			var metadata = new Metadata();

			addClientName(ctxt, metadata);
			addRequestId(ctxt, metadata);
			Optional.ofNullable(ctxt.getUserId())
					.map(OzgCloudUserId::toString)
					.ifPresent(userId -> addToMetadata(metadata, KEY_USER_ID, userId));
			Optional.ofNullable(ctxt.getAccessLimitedToOrgaIds())
					.ifPresent(ids -> ids.stream().forEach(id -> addToMetadata(metadata, KEY_ACCESS_LIMITED_ORGAID, id)));
			Optional.ofNullable(ctxt.isAccessLimited()).map(Boolean::valueOf)
					.ifPresent(limited -> addToMetadata(metadata, KEY_ACCESS_LIMITED, limited.toString()));

			return metadata;
		}

		private void addRequestId(CallContext ctxt, Metadata metadata) {
			Optional.ofNullable(ctxt.getRequestId()).ifPresentOrElse(reqId -> metadata.put(createKeyOf(KEY_REQUEST_ID), reqId.getBytes()),
					() -> addToMetadata(metadata, KEY_REQUEST_ID, UUID.randomUUID().toString()));
		}

		private void addClientName(CallContext ctxt, Metadata metadata) {
			Optional.ofNullable(ctxt.getClientName())
					.filter(StringUtils::isNoneBlank)
					.ifPresentOrElse(clientName -> addToMetadata(metadata, KEY_CLIENT_NAME, clientName),
							() -> {
								throw new IllegalStateException("CLIENT_NAME must not be blank");
							});
		}

		private void addToMetadata(Metadata metadata, String key, String value) {
			metadata.put(createKeyOf(key), value.getBytes());
		}

	}
}
