/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.apilib.file.grpc;

import java.io.OutputStream;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.TimeoutException;
import java.util.logging.Level;

import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.stereotype.Service;

import de.ozgcloud.apilib.common.callcontext.OzgCloudCallContextAttachingInterceptor;
import de.ozgcloud.apilib.common.callcontext.OzgCloudCallContextProvider;
import de.ozgcloud.apilib.common.errorhandling.NotFoundException;
import de.ozgcloud.apilib.file.OzgCloudFile;
import de.ozgcloud.apilib.file.OzgCloudFileId;
import de.ozgcloud.apilib.file.OzgCloudFileService;
import de.ozgcloud.common.binaryfile.FileId;
import de.ozgcloud.common.errorhandling.TechnicalException;
import de.ozgcloud.vorgang.grpc.binaryFile.BinaryFileServiceGrpc.BinaryFileServiceBlockingStub;
import de.ozgcloud.vorgang.grpc.binaryFile.BinaryFileServiceGrpc.BinaryFileServiceStub;
import de.ozgcloud.vorgang.grpc.binaryFile.GrpcBinaryFilesRequest;
import de.ozgcloud.vorgang.grpc.binaryFile.GrpcFindFilesResponse;
import de.ozgcloud.vorgang.grpc.binaryFile.GrpcGetBinaryFileDataRequest;
import de.ozgcloud.vorgang.grpc.file.GrpcOzgFile;
import lombok.RequiredArgsConstructor;
import lombok.extern.java.Log;
import net.devh.boot.grpc.client.inject.GrpcClient;

@Service
@ConditionalOnProperty("ozgcloud.file-manager.address")
@RequiredArgsConstructor
@Log
public class GrpcOzgCloudFileService implements OzgCloudFileService {

	@GrpcClient("file-manager")
	private final BinaryFileServiceBlockingStub blockingStub;
	@GrpcClient("file-manager")
	private final BinaryFileServiceStub asyncServiceStub;

	private final OzgCloudCallContextProvider contextProvider;

	private final OzgCloudFileMapper mapper;

	static final int CHUNK_SIZE = 255 * 1024;

	@Override
	public OzgCloudFile getFile(OzgCloudFileId id) {
		var response = getBlockingStub().findBinaryFilesMetaData(buildFindFileRequest(id));

		return mapper.fromGrpc(getSingleResponse(response, id));
	}

	GrpcBinaryFilesRequest buildFindFileRequest(OzgCloudFileId id) {
		return GrpcBinaryFilesRequest.newBuilder()
				.addFileId(id.toString())
				.build();
	}

	GrpcOzgFile getSingleResponse(GrpcFindFilesResponse response, OzgCloudFileId id) {
		var responseCount = response.getFileCount();
		if (responseCount == 0) {
			throw new NotFoundException(id, "OzgCloudFile");
		}
		if (responseCount > 1) {
			LOG.log(Level.WARNING, () -> "Found more then one file with id '%s'. Returning first one, only.".formatted(id.toString()));
		}
		return response.getFile(0);
	}

	@Override
	public void writeFileDataToStream(OzgCloudFileId id, OutputStream streamToWriteData) {
		var fileId = FileId.from(id.toString());
		var streamFuture = new CompletableFuture<Boolean>();
		var responseObserver = new OzgCloudFileDownloadStreamObserver(streamFuture, streamToWriteData);

		getAsyncServiceStub().getBinaryFileContent(buildGrpcGetBinaryFileDataRequest(fileId), responseObserver);

		waitUntilFutureToComplete(streamFuture);
	}

	private GrpcGetBinaryFileDataRequest buildGrpcGetBinaryFileDataRequest(FileId fileId) {
		return GrpcGetBinaryFileDataRequest.newBuilder().setFileId(fileId.toString()).build();
	}

	void waitUntilFutureToComplete(CompletableFuture<Boolean> streamFuture) {
		try {
			streamFuture.get(10, TimeUnit.MINUTES);
		} catch (InterruptedException e) {
			Thread.currentThread().interrupt();
			throw new TechnicalException(e.getMessage(), e);
		} catch (ExecutionException | TimeoutException e) {
			throw new TechnicalException(e.getMessage(), e);
		}
	}

	BinaryFileServiceBlockingStub getBlockingStub() {
		return blockingStub.withInterceptors(new OzgCloudCallContextAttachingInterceptor(contextProvider));
	}

	BinaryFileServiceStub getAsyncServiceStub() {
		return asyncServiceStub.withInterceptors(new OzgCloudCallContextAttachingInterceptor(contextProvider));
	}
}
