/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.apilib.vorgang.grpc;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.ArgumentCaptor;
import org.mockito.Captor;
import org.mockito.InjectMocks;
import org.mockito.Mock;

import de.ozgcloud.apilib.vorgang.OzgCloudVorgangStubTestFactory;
import de.ozgcloud.apilib.vorgang.OzgCloudVorgangTestFactory;
import de.ozgcloud.apilib.vorgang.Page;
import de.ozgcloud.vorgang.vorgang.GrpcFindVorgangRequest;
import de.ozgcloud.vorgang.vorgang.GrpcFindVorgangResponse;
import de.ozgcloud.vorgang.vorgang.GrpcFindVorgangWithEingangRequest;
import de.ozgcloud.vorgang.vorgang.GrpcVorgangHeader;
import de.ozgcloud.vorgang.vorgang.VorgangServiceGrpc.VorgangServiceBlockingStub;

class GrpcOzgCloudVorgangServiceTest {

	@InjectMocks
	private GrpcOzgCloudVorgangService service;

	@Mock
	private VorgangServiceBlockingStub stub;
	@Mock
	private OzgCloudVorgangMapper mapper;
	@Mock
	private OzgCloudVorgangStubMapper stubMapper;

	@Nested
	class TestGetById {
		@Captor
		private ArgumentCaptor<GrpcFindVorgangWithEingangRequest> requestCaptor;

		@BeforeEach
		void init() {
			when(stub.withInterceptors(any())).thenReturn(stub);
			when(stub.findVorgangWithEingang(any())).thenReturn(GrpcFindVorgangWithEingangResponseTestFactory.create());
		}

		@Test
		void shouldCallStub() {
			service.getById(OzgCloudVorgangTestFactory.ID);

			verify(stub).findVorgangWithEingang(notNull());
		}

		@Test
		void shouldHaveVorgangId() {
			service.getById(OzgCloudVorgangTestFactory.ID);

			verify(stub).findVorgangWithEingang(requestCaptor.capture());
			assertThat(requestCaptor.getValue().getId()).isEqualTo(OzgCloudVorgangTestFactory.ID.toString());
		}

		@Test
		void shouldMapResponse() {
			service.getById(OzgCloudVorgangTestFactory.ID);

			verify(mapper).fromGrpc(any());
		}

		@Test
		void shouldReturnMappedVorgang() {
			var vorgang = OzgCloudVorgangTestFactory.create();
			when(mapper.fromGrpc(any())).thenReturn(vorgang);

			var result = service.getById(OzgCloudVorgangTestFactory.ID);

			assertThat(result).isSameAs(vorgang);
		}
	}

	@Nested
	class TestSearch {

		private static final String SEARCH_STR = "search";

		@Captor
		private ArgumentCaptor<GrpcFindVorgangRequest> requestCaptor;

		@Nested
		class BuildSearchRequest {
			private static final Page PAGE = Page.builder().limit(40).offset(21).build();

			@Test
			void shouldSetSearchString() {
				var request = service.buildSearchRequest(SEARCH_STR, PAGE);

				assertThat(request.getSearchBy()).isEqualTo(SEARCH_STR);
			}

			@Test
			void shouldSetLimit() {
				var request = service.buildSearchRequest(SEARCH_STR, PAGE);

				assertThat(request.getLimit()).isEqualTo(40);
			}

			@Test
			void shouldSetOffset() {
				var request = service.buildSearchRequest(SEARCH_STR, PAGE);

				assertThat(request.getOffset()).isEqualTo(21);
			}
		}

		@Nested
		class doingRequest {

			private static final GrpcVorgangHeader VORGANG = GrpcVorgangHeaderTestFactory.create();
			private static final GrpcFindVorgangResponse RESPONSE = GrpcFindVorgangResponse.newBuilder()
					.addVorgang(VORGANG).build();

			@BeforeEach
			void init() {
				when(stub.withInterceptors(any())).thenReturn(stub);
				when(stub.findVorgang(any())).thenReturn(RESPONSE);
			}

			@Test
			void shouldCallStub() {
				service.search(SEARCH_STR);

				verify(stub).findVorgang(notNull());
			}

			@Test
			void shouldMapResponse() {
				service.search(SEARCH_STR);

				verify(stubMapper).fromGrpc(notNull());
			}

			@Test
			void shouldReturnResult() {
				var vorgang = OzgCloudVorgangStubTestFactory.create();
				when(stubMapper.fromGrpc(any())).thenReturn(vorgang);

				var result = service.search(SEARCH_STR);

				assertThat(result).contains(vorgang);
			}
		}

	}
}
